@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of mailmanclient.
 #
@@ -16,8 +16,6 @@
 
 """Client code."""
 
-from __future__ import absolute_import, unicode_literals
-
 import warnings
 from operator import itemgetter
 
@@ -50,20 +48,51 @@ class Client:
     """Access the Mailman REST API root.
 
     :param baseurl: The base url to access the Mailman 3 REST API.
+    :type baseurl: str
     :param name: The Basic Auth user name.  If given, the `password` must
         also be given.
+    :type name: str
     :param password: The Basic Auth password.  If given the `name` must
         also be given.
+    :type password: str
+    :param request_hooks: Callable hooks to process request parameters before
+        being sent to Core's API.
+    :type request_hooks: List[callables]
     """
 
-    def __init__(self, baseurl, name=None, password=None):
+    def __init__(self, baseurl, name=None, password=None, request_hooks=None):
         """Initialize client access to the REST API."""
-        self._connection = Connection(baseurl, name, password)
+        self._connection = Connection(baseurl, name, password, request_hooks)
 
     def __repr__(self):
         return '<Client ({0.name}:{0.password}) {0.baseurl}>'.format(
             self._connection)
 
+    def add_hooks(self, request_hooks):
+        """Add a hook to process connections to Mailman's API.
+
+        Hooks are callables that are passed in the parameters of HTTP call made
+        to Mailman Core' RESt API and are expected to return the same number of
+        parameters back. This can be useful to manipulate parameters and update
+        them, or simply log each call to API.
+        ::
+
+            from mailmanclient.client import Client
+            client = Client('http://localhost:8001/3.1', <user>, <password>)
+
+            def sample_hook(params):
+                print(f'Request params are {params}')
+                return params
+
+            client.add_hook([sample_hook])
+
+
+        :param request_hooks: A list of callables that take in a dictionary of
+           parameters. ``params`` is the list of request parameters before the
+           request is made.
+        """
+        self._connection.add_hooks(request_hooks=request_hooks)
+
     @property
     def system(self):
         """Get the basic system information.
@@ -442,3 +471,29 @@ class Client:
         return [MailingList(self._connection, entry['self_link'], entry)
                 for entry in content['entries']
                 if not mail_host or entry['mail_host'] == mail_host]
+
+    def find_users(self, query):
+        """Find users with query string matching display name and emails.
+
+        :param str query: The string to search for. The search string is case
+            insensitive as core doens't care about the case.
+        :param int count: Number of entries per-page (defaults to 50).
+        :param int page: The page number to return (defaults to 1).
+        """
+        url = 'users/find?q={}'.format(query)
+        response, content = self._connection.call(url)
+        if 'entries' not in content:
+            return []
+        return [User(self._connection, entry['self_link'], entry)
+                for entry in content['entries']]
+
+    def find_users_page(self, query, count, page):
+        """Same as :py:meth:`find_users` but allows for pagination.
+
+        :param str query: The string to search for. The search string is case
+            insensitive as core doens't care about the case.
+        :param int count: Number of entries per-page (defaults to 50).
+        :param int page: The page number to return (defaults to 1).
+        """
+        url = 'users/find?q={}'.format(query)
+        return Page(self._connection, url, User, count, page)
