@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of mailmanclient.
 #
@@ -48,7 +48,32 @@ class MailingList(RESTObject):
 
     @property
     def owners(self):
-        url = self._url + '/roster/owner'
+        """All MailingList owners."""
+        return self.get_roster('owner')
+
+    def get_roster(self, roster, fields=None):
+        """Get roster of the MailingList.
+
+        If the fields is specified without `self_link` and `address`, they are
+        added since it is required for returning the response.
+
+        :param str roster: One of the Membership rosters from
+           'owner', 'moderator', 'member' and 'nonmember'.
+        :param List[str] fields: List of Member's fields to fetch from the
+           API. Skipping certain fields can speed up the API response
+           when they aren't required since they need to be fetched from
+           database individually.
+        """
+        url = self._url + '/roster/{}'.format(roster)
+        if fields is not None:
+            # We cannot instantiate Member object without address and
+            # self_link objects, so just add them too. They don't add
+            # a lot of overhead.
+            if 'address' not in fields:
+                fields.append('address')
+            if 'self_link' not in fields:
+                fields.append('self_link')
+            url += '?' + '&'.join('fields={}'.format(each) for each in fields)
         response, content = self._connection.call(url)
         if 'entries' not in content:
             return []
@@ -59,40 +84,41 @@ class MailingList(RESTObject):
 
     @property
     def moderators(self):
-        url = self._url + '/roster/moderator'
-        response, content = self._connection.call(url)
-        if 'entries' not in content:
-            return []
-        else:
-            return [Member(self._connection, entry['self_link'], entry)
-                    for entry in sorted(content['entries'],
-                                        key=itemgetter('address'))]
+        """All MailingList moderators."""
+        return self.get_roster('moderator')
 
     @property
     def members(self):
-        url = 'lists/{0}/roster/member'.format(self.fqdn_listname)
-        response, content = self._connection.call(url)
-        if 'entries' not in content:
-            return []
-        return [Member(self._connection, entry['self_link'], entry)
-                for entry in sorted(content['entries'],
-                                    key=itemgetter('address'))]
+        """All MailingList members."""
+        return self.get_roster('member')
 
     @property
     def nonmembers(self):
-        url = 'lists/{0}/roster/nonmember'.format(self.fqdn_listname)
-        response, content = self._connection.call(url)
-        if 'entries' not in content:
-            return []
-        return [Member(self._connection, entry['self_link'], entry)
-                for entry in sorted(content['entries'],
-                                    key=itemgetter('address'))]
+        """All MailingList non-members."""
+        return self.get_roster('nonmember')
 
-    def get_member_page(self, count=50, page=1):
+    def get_member_page(self, count=50, page=1, fields=None):
+        """Return a paginated list of MailingList's members.
+
+        :param int count: Count of members in one page.
+        :param int page: The page number.
+        """
         url = 'lists/{0}/roster/member'.format(self.fqdn_listname)
         return Page(self._connection, url, Member, count, page)
 
-    def find_members(self, address=None, role=None, page=None, count=50):
+    def find_members(
+            self, address=None, role=None, page=None, count=50):
+        """Find a Mailinglist's members.
+
+        This provides a filtering API for list's Members including,
+        non-members, owners and moderators by speciying the role.
+
+        :param str address: Member's address.
+        :param str role: Member's role.
+        :param int page: Page number for paginated results.
+        :param int count: Number of results per-page for paginated results.
+
+        """
         data = {'list_id': self.list_id}
         if address:
             data['subscriber'] = address
@@ -111,6 +137,7 @@ class MailingList(RESTObject):
 
     @property
     def settings(self):
+        """All MailingList settings."""
         if self._settings is None:
             self._settings = Settings(
                 self._connection,
@@ -119,7 +146,7 @@ class MailingList(RESTObject):
 
     @property
     def held(self):
-        """Return a list of dicts with held message information."""
+        """Held messages of a MailingList.."""
         response, content = self._connection.call(
             'lists/{0}/held'.format(self.fqdn_listname), None, 'GET')
         if 'entries' not in content:
@@ -128,16 +155,25 @@ class MailingList(RESTObject):
                 for entry in content['entries']]
 
     def get_held_page(self, count=50, page=1):
+        """Paginated list of held messages for the MailingList.
+
+        :param int page: Page number for paginated results.
+        :param int count: Number of results per-page for paginated results.
+        """
         url = 'lists/{0}/held'.format(self.fqdn_listname)
         return Page(self._connection, url, HeldMessage, count, page)
 
     def get_held_count(self):
-        """Get a count of held messages."""
+        """Get a count of held messages for the MailingList."""
         response, json = self._connection.call(
             'lists/{}/held/count'.format(self.fqdn_listname), None, 'GET')
         return json['count']
 
     def get_held_message(self, held_id):
+        """Get a single held message for MailingList.
+
+        :param int held_id: Held message id to get.
+        """
         url = 'lists/{0}/held/{1}'.format(self.fqdn_listname, held_id)
         return HeldMessage(self._connection, url)
 
@@ -146,7 +182,12 @@ class MailingList(RESTObject):
         """See :meth:`get_requests`."""
         return self.get_requests()
 
-    def get_requests(self, token_owner=None):
+    @property
+    def unsubscription_requests(self):
+        """Get a list of subscription requests pending moderator Approval."""
+        return self.get_requests(request_type='unsubscription')
+
+    def get_requests(self, token_owner=None, request_type='subscription'):
         """Return a list of dicts with subscription requests.
 
         This is the new API for requests which allows filtering via
@@ -155,10 +196,17 @@ class MailingList(RESTObject):
 
         :param token_owner: Who owns the pending requests?  Should be one in
             'no_one', 'moderator' and 'subscriber'.
+        :param request_type: The type of pending request. Value should be in
+            'subscription' or 'unsubscription'. Defaults to 'subscription'.
         """
         url = 'lists/{0}/requests'.format(self.fqdn_listname)
+        fragments = []
         if token_owner:
-            url += '?token_owner={}'.format(token_owner)
+            fragments.append('token_owner={}'.format(token_owner))
+        if request_type:
+            fragments.append('request_type={}'.format(request_type))
+        if fragments:
+            url += '?{}'.format('&'.join(fragments))
         response, content = self._connection.call(url, None, 'GET')
         if 'entries' not in content:
             return []
@@ -202,6 +250,7 @@ class MailingList(RESTObject):
 
     @property
     def archivers(self):
+        """Get a list of MailingList archivers."""
         url = 'lists/{0}/archivers'.format(self.list_id)
         return ListArchivers(self._connection, url, self)
 
@@ -213,12 +262,28 @@ class MailingList(RESTObject):
         archivers.save()
 
     def add_owner(self, address, display_name=None):
+        """Add a list owner.
+
+        :param str address: Email address of the owner.
+        :param str display_name: Display name of the Owner.
+        """
         self.add_role('owner', address, display_name)
 
     def add_moderator(self, address, display_name=None):
+        """Add a list moderator.
+
+        :param str address: Email address of the moderator.
+        :param str display_name: Display name of the moderator.
+        """
         self.add_role('moderator', address, display_name)
 
     def add_role(self, role, address, display_name=None):
+        """Add a new Member with a specific role.
+
+        :param str role: The role for the new member.
+        :param str address: A valid email address for the new Member.
+        :param str display_name: An optional display name for the Member.
+        """
         data = dict(list_id=self.list_id,
                     subscriber=address,
                     display_name=display_name,
@@ -226,12 +291,25 @@ class MailingList(RESTObject):
         self._connection.call('members', data)
 
     def remove_owner(self, address):
+        """Remove a list owner.
+
+        :param str address: Email address of the owner to remove.
+        """
         self.remove_role('owner', address)
 
     def remove_moderator(self, address):
+        """Remove a list moderator.
+
+        :param str address: Email address of the moderator to remove.
+        """
         self.remove_role('moderator', address)
 
     def remove_role(self, role, address):
+        """Remove a list Member with a specific Role.
+
+        :param str role: The role for the new member.
+        :param str address: A valid email address for the new Member.
+        """
         url = 'lists/%s/%s/%s' % (
             self.fqdn_listname, role, quote_plus(address))
         self._connection.call(url, method='DELETE')
@@ -285,15 +363,20 @@ class MailingList(RESTObject):
         """
         return self.moderate_message(request_id, 'accept')
 
-    def moderate_request(self, request_id, action):
+    def moderate_request(self, request_id, action, reason=None):
         """
         Moderate a subscription request.
 
         :param action: accept|reject|discard|defer
         :type action: str.
+        :param reason: The reason associated with rejections.
+        :type reason: str
         """
         path = 'lists/{0}/requests/{1}'.format(self.list_id, request_id)
-        response, content = self._connection.call(path, {'action': action})
+        data = {'action': action}
+        if reason:
+            data['reason'] = reason
+        response, content = self._connection.call(path, data)
         return response
 
     def manage_request(self, token, action):
@@ -321,7 +404,7 @@ class MailingList(RESTObject):
         return self.moderate_request(request_id, 'defer')
 
     def _get_membership(self, email, role):
-        """Get a membership.
+        """Get a single membership resource.
 
         :param address: The email address of the member for this list.
         :param role: The membership role.
@@ -340,7 +423,7 @@ class MailingList(RESTObject):
                              (email, role, self.fqdn_listname))
 
     def get_member(self, email):
-        """Get a membership.
+        """Get a Member of the list.
 
         :param address: The email address of the member for this list.
         :return: A member proxy object.
@@ -357,7 +440,8 @@ class MailingList(RESTObject):
 
     def subscribe(self, address, display_name=None, pre_verified=False,
                   pre_confirmed=False, pre_approved=False, invitation=False,
-                  send_welcome_message=None):
+                  send_welcome_message=None, delivery_mode=None,
+                  delivery_status=None):
         """Subscribe an email address to a mailing list.
 
         :param address: Email address to subscribe to the list.
@@ -374,6 +458,12 @@ class MailingList(RESTObject):
         :type invitation: bool
         :param send_welcome_message: True if welcome message should be sent.
         :type send_welcome_message: bool
+        :param delivery_mode: Delivery mode of the Member.
+        :type delivery_mode: str. One between 'regular', 'plaintext_digests',
+            'mime_digests', 'summary_digests'.
+        :param delivery_status: Delivery status of the Member.
+        :type delivery_status: str. One between 'enabled', 'by_owner',
+            'by_moderator', 'by_user'.
         :return: A member proxy object.
         """
         data = dict(
@@ -390,6 +480,10 @@ class MailingList(RESTObject):
             data['pre_approved'] = True
         if invitation:
             data['invitation'] = True
+        if delivery_mode:
+            data['delivery_mode'] = delivery_mode
+        if delivery_status:
+            data['delivery_status'] = delivery_status
         # Even if it is False, we should send this value because it means we
         # should suppress welcome message, so check for None value to skip the
         # parameter.
@@ -404,16 +498,29 @@ class MailingList(RESTObject):
         # is returned.
         return Member(self._connection, response.headers.get('location'))
 
-    def unsubscribe(self, email):
+    def unsubscribe(self, email, pre_confirmed=None,
+                    pre_approved=None):
         """Unsubscribe an email address from a mailing list.
 
-        :param address: The address to unsubscribe.
+        :param address: Email address to unsubscribe.
+        :type address: str
+        :param pre_confirmed: True if unsubscribe is approved by the user.
+        :type pre_confirmed: bool
+        :param pre_approved: True if unsubscribe is moderator-approved.
+        :type pre_approved: bool
         """
+        data = dict()
+        if pre_confirmed is not None:
+            data['pre_confirmed'] = pre_confirmed
+        if pre_approved is not None:
+            data['pre_approved'] = pre_approved
         try:
             path = 'lists/{0}/member/{1}'.format(self.list_id, email)
-            self._connection.call(path, method='DELETE')
+            response, json = self._connection.call(path, data, method='DELETE')
+            if response.status_code == 202:
+                return json
         except HTTPError:
-            # The member link does not exist, i.e. he is not a member
+            # The member link does not exist, i.e. they are not a member
             raise ValueError('%s is not a member address of %s' %
                              (email, self.fqdn_listname))
 
@@ -435,26 +542,41 @@ class MailingList(RESTObject):
 
     @property
     def bans(self):
+        """A list of banned addresses for this MailingList."""
         from mailmanclient.restobjects.ban import Bans
         url = 'lists/{0}/bans'.format(self.list_id)
         return Bans(self._connection, url, mlist=self)
 
     def get_bans_page(self, count=50, page=1):
+        """Get a paginated list of bans for this MailingList.
+
+        :param int page: Page number for paginated results.
+        :param int count: Number of results per-page for paginated results.
+        """
         from mailmanclient.restobjects.ban import BannedAddress
         url = 'lists/{0}/bans'.format(self.list_id)
         return Page(self._connection, url, BannedAddress, count, page)
 
     @property
     def header_matches(self):
+        """A list of header-match rules for the MailingList."""
         url = 'lists/{0}/header-matches'.format(self.list_id)
         return HeaderMatches(self._connection, url, self)
 
     @property
     def templates(self):
+        """Get a list of MailingList templates."""
         url = self._url + '/uris'
         return TemplateList(self._connection, url)
 
     def set_template(self, template_name, uri, username=None, password=None):
+        """Set a MailingList template URI.
+
+        :param str template_name: The name of the template.
+        :param str uri: The URI to fetch the template.
+        :param str username: Username for fetching template from uri.
+        :param str password: Password for fetching template from uri.
+        """
         url = self._url + '/uris'
         data = {template_name: uri}
         if username is not None and password is not None:
